@@ -15,11 +15,6 @@
 #define DOSLIKE 1
 #endif
 
-/* System V or system BSD? */
-#if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD)
-#define SYSBSD 1
-#endif
-
 /* seems like everybody needs these header files */
 #include <sys/types.h>
 #include <ctype.h>
@@ -37,10 +32,6 @@
 #include <unistd.h>
 #endif
 
-#include "tcp.h"
-
-#include <jsapi.h>
-
 #ifndef O_BINARY
 #define O_BINARY 0
 #endif
@@ -51,16 +42,6 @@
 #define O_SYNC 0
 #endif
 
-/* Some error constants that aren't defined in older versions of libcurl. 
- * I'm not too worried about them, because the catch-all handles so many
- * things..  Plus, everyone will eventually be using the latest libcurl. */
-#ifndef CURLE_PEER_FAILED_VERIFICATION
-#define CURLE_PEER_FAILED_VERIFICATION -1
-#endif
-#ifndef CURLE_REMOTE_ACCESS_DENIED
-#define CURLE_REMOTE_ACCESS_DENIED -2
-#endif
-
 /* WARNING:  the following typedef is pseudo-standard in C.
  * Some systems will define ushort in sys/types.h, others will not.
  * Unfortunately there is no #define symbol to key on;
@@ -75,11 +56,14 @@ typedef unsigned long ulong;
 typedef unsigned char uchar;
 
 /* We use uchar for boolean fields. */
-typedef uchar bool;
-#define true 1
-#define false 0
+/* The type is eb_bool, an edbrowse bool, so as not to conflict with C++. */
+typedef uchar eb_bool;
+#define eb_true 1
+#define eb_false 0
 
-typedef ushort idNameCode;
+typedef unsigned int IP32bit;
+#define NULL_IP (IP32bit)(-1)
+#define ABSPATH 256
 
 #define stringEqual !strcmp
 
@@ -115,7 +99,7 @@ typedef uchar *pst;		/* perl string */
 #define MAXRE 400
 /* How long can an entered line be? */
 #define MAXTTYLINE 256
-/* Longest line that can take the substitute command */
+/* Longest line that can take the breakline command */
 #define REPLACELINELEN 50000
 /* The longest string, in certain contexts. */
 #define MAXSTRLEN 1024
@@ -125,9 +109,11 @@ typedef uchar *pst;		/* perl string */
 #define MAXACCOUNT 100
 /* Number of mime types */
 #define MAXMIME 40
+/* Number of proxy entries */
+#define MAXPROXY 200
 /* number of db tables */
 #define MAXDBT 100
-#define MAXTCOLS 20
+#define MAXTCOLS 40
 /* How many sessions open concurrently */
 #define MAXSESSION 1000
 /* Allocation increment for a growing string, that we don't expect
@@ -163,83 +149,92 @@ extern char EMPTYSTRING[];	/* use this whenever you would use "" */
  * Anyways, this array holds up to 10 user agent strings. */
 extern char *userAgents[10], *currentAgent;
 
-struct MACCOUNT {		/* pop3 account */
-    char *login, *password, *from, *reply;
-    char *inurl, *outurl;
-    int inport, outport;
-    uchar inssl, outssl;
-    char nofetch;
+struct eb_curl_callback_data {
+	char **buffer;
+	int *length;
 };
 
+struct MACCOUNT {		/* pop3 account */
+	char *login, *password, *from, *reply;
+	char *inurl, *outurl;
+	int inport, outport;
+	uchar inssl, outssl;
+	char nofetch;
+};
+extern struct MACCOUNT accounts[];	/* all the email accounts */
+extern int maxAccount;		/* how many email accounts specified */
+
+struct PXENT { /* proxy entry */
+	char *prot; /* null means any */
+	char *domain; /* null means any */
+	char *proxy; /* null means direct */
+};
+extern struct PXENT proxyEntries[];
+extern int maxproxy;
+
 struct MIMETYPE {
-    char *type, *desc;
-    char *suffix, *prot, *program;
-    bool stream;
+	char *type, *desc;
+	char *suffix, *prot, *program;
+	eb_bool stream;
 };
+extern struct MIMETYPE mimetypes[];
+extern int maxMime;		/* how many mime types specified */
 
 struct DBTABLE {
-    char *name, *shortname;
-    char *cols[MAXTCOLS];
-    char ncols;
-    unsigned char key1, key2, key3, key4;
-    char *types;
-    char *nullable;
+	char *name, *shortname;
+	char *cols[MAXTCOLS];
+	char ncols;
+	unsigned char key1, key2, key3, key4;
+	char *types;
+	char *nullable;
 };
 
 /* various globals */
 extern CURL *curl_handle;
 extern int debugLevel;		/* 0 to 9 */
+extern char *sslCerts;		/* ssl certificates to validate the secure server */
+extern int verifyCertificates;	/* is a certificate required for the ssl connection? */
 extern int displayLength;	/* when printing a line */
+extern int jsPool;		/* size of js pool in megabytes */
 extern int webTimeout, mailTimeout;
 extern int browseLine;		/* line number, for error reporting */
-extern bool ismc;		/* Is the program running as a mail client? */
-extern bool cons_utf8;		/* does the console expect utf8? */
-extern bool iuConvert;		/* perform iso utf8 conversions automatically */
+extern eb_bool sqlPresent;	/* Was edbrowse compiled with SQL built in? */
+extern eb_bool ismc;		/* Is the program running as a mail client? */
+extern int eb_lang;		/* edbrowse language, determined by $LANG */
+extern eb_bool cons_utf8;	/* does the console expect utf8? */
+extern eb_bool iuConvert;	/* perform iso utf8 conversions automatically */
 extern char type8859;		/* 1 through 15 */
-extern bool js_redirects;	/* window.location = new_url */
+extern eb_bool js_redirects;	/* window.location = new_url */
 extern uchar browseLocal;	/* browsing a local file */
-extern bool parsePage;		/* parsing the html page, and any java therein */
-extern bool htmlAttrVal_nl;	/* allow nl in the attribute of an html tag */
-extern bool passMail;		/* pass mail across the filters */
-extern bool errorExit;		/* exit on any error, for scripting purposes */
-extern bool isInteractive;
-extern volatile bool intFlag;	/* set this when interrupt signal is caught */
-extern bool binaryDetect;
-extern bool inputReadLine;
-extern bool listNA;		/* list nonascii chars */
-extern bool inInput;		/* reading line from standard in */
+extern eb_bool parsePage;	/* parsing the html page and any js therein */
+extern eb_bool htmlAttrVal_nl;	/* allow nl in the attribute of an html tag */
+extern eb_bool passMail;	/* pass mail across the filters */
+extern eb_bool errorExit;	/* exit on any error, for scripting purposes */
+extern eb_bool isInteractive;
+extern volatile eb_bool intFlag;	/* set this when interrupt signal is caught */
+extern eb_bool binaryDetect;
+extern eb_bool inputReadLine;
+extern eb_bool listNA;		/* list nonascii chars */
+extern eb_bool inInput;		/* reading line from standard in */
 extern int fileSize;		/* when reading/writing files */
-extern int maxFileSize;		/* max size of an editable file */
 extern int mssock;		/* mail server socket */
-extern int hcode;		/* http code, like 404 file not found */
-extern char herror[];		/* html error */
+extern long hcode;		/* http code, like 404 file not found */
 extern char errorMsg[];		/* generated error message */
 extern char serverLine[];	/* lines to and from the mail server */
-extern JSContext *jcx;		/* javascript context */
-extern JSObject *jwin;		/* javascript window object */
-extern JSObject *jdoc;		/* javascript document object */
-extern JSObject *jwloc;		/* javascript window.location */
-extern JSObject *jdloc;		/* javascript document.location */
-extern int maxAccount;		/* how many email accounts specified */
 extern int localAccount;	/* this is the smtp server for outgoing mail */
 extern char *mailDir;		/* move to this directory when fetching mail */
 extern char *mailUnread;	/* place to hold fetched but unread mail */
 /* Keep a copy of unformatted mail that you probably won't need again,
  * but you never know. Should probably live somewhere under .Trash */
 extern char *mailStash;
-extern struct MACCOUNT accounts[];	/* all the email accounts */
-extern int maxMime;		/* how many mime types specified */
-extern struct MIMETYPE mimetypes[];
 extern char *dbarea, *dblogin, *dbpw;	/* to log into the database */
-extern bool fetchBlobColumns;
-extern char *proxy_host;
-extern bool caseInsensitive, searchStringsAll;
-extern bool undoable;		/* an undoable operation is taking place */
-extern bool allowRedirection;	/* from http code 301, or http refresh */
-extern bool sendReferrer;	/* in the http header */
-extern bool allowJS;		/* javascript on */
-extern bool helpMessagesOn;	/* no need to type h */
-extern bool showHiddenFiles;	/* during directory scan */
+extern eb_bool fetchBlobColumns;
+extern eb_bool caseInsensitive, searchStringsAll;
+extern eb_bool allowRedirection;	/* from http code 301, or http refresh */
+extern eb_bool sendReferrer;	/* in the http header */
+extern eb_bool allowJS;		/* javascript on */
+extern eb_bool helpMessagesOn;	/* no need to type h */
+extern eb_bool showHiddenFiles;	/* during directory scan */
 extern uchar dirWrite;		/* directory write mode, e.g. rename files */
 extern uchar endMarks;		/* do we print ^ $ at the start and end of lines? */
 extern int context;		/* which session (buffer) are we in? */
@@ -260,53 +255,9 @@ extern char *edbrowseTempFile;
 extern char *edbrowseTempPDF;
 extern char *edbrowseTempHTML;
 
-/* Ok, this is kinda simple and stupid.
- * Every line in every window in every session is a string, without the
- * newline, and all these strings are managed by an array of pointers.
- * So your particular buffer may have 3 lines in it,
- * and these are represented internally by three numbers, 12 29 and 83.
- * So the text is textLines[12], textLines[29], textLines[83].
- * Now, if you copy a line of text, I actually copy the string.
- * This is a waste I suppose, but then I don't have to worry about
- * what happens if you edit one line and not the other.
- * They're already separate strings, so we're ok. */
-extern pst *textLines;
-extern int textLinesMax, textLinesCount;	/* size of textLines array */
-
-/*********************************************************************
-If a file has 30 lines, it is represented by 30 numbers,
-indexes into textLines[] above.
-Should we use an array of numbers, or a string of numbers
-represented by decimal digits?
-Both are painful, in different ways.
-Consider inserting a block of text, a very common operation.
-In a list, we would have to slide all the following numbers down.
-Granted, that's better than moving all those lines of text,
-but it's still a pain to program, and somewhat inefficient.
-If we use strings, we take the original string of numbers,
-break it at the insert point, and make a new string
-by concatenating these two pieces with the new block.
-The same issues arise when deleting text near the top of a file.
-This and other considerations push me towards strings.
-I currently use chars 0 through 6 for the line number,
-7 for the g// flag,
-and 8 for the directory suffix.
-You know, the slash that we put on the end of a directory,
-or the | on the end of a fifo etc.
-9 is unused, I think.
-Thus the following definitions.
-*********************************************************************/
-
-#define LNMAX 10000000
-#define LNWIDTH 10
-#define LNSPACE "          "
-#define LNFORMAT "%07d   "
-#define LNGLOB 7
-#define LNDIR 8
-
 struct listHead {
-    void *next;
-    void *prev;
+	void *next;
+	void *prev;
 };
 
 /* Macros to loop through the items in a list. */
@@ -317,60 +268,86 @@ struct listHead {
 (e) != (void*)&(l); \
 (e) = ((struct listHead *)e)->prev)
 
+/* Forward declaration, real declaration in js.h. */
+struct ebWindowJSState;
+/* Another forward declaration, opaque outside html.c. */
+struct htmlTag;
+
+/* A pointer to the text of a line, and other line attributes */
+struct lineMap {
+	pst text;
+	char ds1, ds2;		/* directory suffix */
+	eb_bool gflag;		/* for g// */
+	char filler;		/* C is going to pad the structure anyways */
+};
+#define LMSIZE sizeof(struct lineMap)
+
 /* an edbrowse window */
 struct ebWindow {
 /* windows stack up as you open new files or follow hyperlinks.
  * Use the back command to pop the stack.
  * The back command follows this link, which is 0 if you are at the top. */
-    struct ebWindow *prev;
+	struct ebWindow *prev;
 /* This is right out of the source for ed.  Current and last line numbers. */
-    int dot, dol;
+	int dot, dol;
 /* remember dot and dol for the raw text, when in browse mode */
-    int r_dot, r_dol;
-    char *fileName;		/* name of file or url */
-    char *firstURL;		/* before http redirection */
-    char *referrer;
-    char *baseDirName;		/* when scanning a directory */
-    char *ft, *fd, *fk;		/* title, description, keywords */
-    char *mailInfo;
-    char lhs[MAXRE], rhs[MAXRE];	/* remembered substitution strings */
-    char *map;			/* indexes into textLines[] */
-    char *r_map;
+	int r_dot, r_dol;
+	char *fileName;		/* name of file or url */
+	char *firstURL;		/* before http redirection */
+	char *referrer;
+	char *baseDirName;	/* when scanning a directory */
+	char *ft, *fd, *fk;	/* title, description, keywords */
+	char *mailInfo;
+	char lhs[MAXRE], rhs[MAXRE];	/* remembered substitution strings */
+	struct lineMap *map, *r_map;
 /* The labels that you set with the k command, and access via 'x.
  * Basically, that's 26 line numbers.
  * Number 0 means the label is not set. */
-    int labels[26], r_labels[26];
+	int labels[26], r_labels[26];
+/* Next is an array of html tags, generated by the browse command,
+ * and used thereafter for hyperlinks, fill-out forms, etc. */
+	void *tags;
 /* Yeah, these could be bit fields in the structure :1; but who cares. */
-    bool lhs_yes, rhs_yes;
-    bool binMode;		/* binary file */
-    bool nlMode;		/* newline at the end */
-    bool rnlMode;
+	eb_bool lhs_yes, rhs_yes;
+	eb_bool binMode;	/* binary file */
+	eb_bool nlMode;		/* newline at the end */
+	eb_bool rnlMode;
 /* Two text modes; these are incompatible with binMode */
-    bool utf8Mode;
-    bool iso8859Mode;
-    bool browseMode;		/* browsing html */
-    bool changeMode;		/* something has changed in this file */
-    bool dirMode;		/* directory mode */
-    bool firstOpMode;		/* first change has been made, undo is possible */
-    bool jsdead;		/* javascript is dead, for this window */
-    bool sqlMode;		/* accessing a table */
-    char *dw;			/* document.write string */
-    int dw_l;			/* length of the above */
-    void *tags;			/* array of html tags, when browsing */
-    IP32bit *iplist;		/* ip addresses referenced by this page */
-    void *jsc;			/* js context, if in browse mode, and running javascript */
-    struct DBTABLE *table;	/* if in sqlMode */
+	eb_bool utf8Mode;
+	eb_bool iso8859Mode;
+	eb_bool browseMode;	/* browsing html */
+	eb_bool changeMode;	/* something has changed in this file */
+	eb_bool dirMode;	/* directory mode */
+	eb_bool undoable;	/* undo is possible */
+	eb_bool sqlMode;	/* accessing a table */
+	char *dw;		/* document.write string */
+	int dw_l;		/* length of the above */
+	struct ebWindowJSState *jss;
+eb_bool js_failed;
+	IP32bit *iplist;	/* ip addresses referenced by this page */
+	struct DBTABLE *table;	/* if in sqlMode */
 };
 extern struct ebWindow *cw;	/* current window */
 
+/*********************************************************************
+Temporary cap on the number of lines, so the integer index into cw->map
+doesn't overflow. This is basically signed int over LMSIZE.
+The former is 2^31 on most machines,
+the latter is at most 12 on a 64-bit machine.
+If ints are larger then I don't even use this constant.
+*********************************************************************/
+
+#define MAXLINES 170000000
+
+#define isJSAlive (cw->jss != NULL && allowJS  && !cw->js_failed)
+
 /* An edit session */
 struct ebSession {
-    struct ebWindow *fw, *lw;	/* first window, last window */
+	struct ebWindow *fw, *lw;	/* first window, last window */
 };
 extern struct ebSession sessionList[];
 extern struct ebSession *cs;	/* current session */
 
-
 /* function prototypes */
 #include "eb.p"
 
